/*!
 * @module vcui.ui.MonthCalendar
 * @license MIT License
 * @description MonthCalendar 컴포넌트
 * @copyright VinylC UID Group
 */
define('ui/monthCalendar', ['jquery', 'vcui', 'libs/jquery.transit', 'helper/gesture'], function ($, core) {
    "use strict";

    var prefix = ".ui_monthCalendar_";
    var dateUtil = core.date;

    var MonthCalendar = core.ui('MonthCalendar', {
        bindjQuery: 'monthCalendar',
        defaults: {
            startDate: null,
            selectDate: null,
            target: null
        },
        selectors: {
            layer: prefix + 'layer',
            list: prefix + 'month > ul'
        },
        /**
         * 생성자
         * @param el
         * @param options
         */
        initialize: function initialize(el, options) {
            var self = this;

            if (self.supr(el, options) === false) {
                return;
            }

            self._parseData();

            self.startDate = self.dates[0];
            self.currentDate = self.dates[self.dates.length - 1];
            self.selectDate = core.date.parse(self.options.selectDate + '-01');
            self.originalDate = core.clone(self.selectDate);

            self._bindEvents();
            self._updateButtons();
            //self.select();
        },
        /**
         * data를 파싱
         * @private
         */
        _parseData: function () {
            var self = this;

            self.dates = core.array.map(self.options.data, function (item) {
                var pairs = item.split('-');

                return new Date(parseInt(pairs[0]), parseInt(pairs[1]) - 1);
            }).sort(function (a, b) {
                if (a.getTime() > b.getTime()) {
                    return 1;
                }
                if (a.getTime() < b.getTime()) {
                    return -1;
                }
                return 0;
            });
        },
        /**
         * 이벤트 바인딩
         * @private
         */
        _bindEvents: function _bindEvents() {
            var self = this;

            // 상단 버튼 클릭시 레이어 표시
            self.on('click', prefix + 'toggle', function (e) {
                e.preventDefault();

                self.selectDate = core.clone(self.originalDate);
                self.currentDate = core.clone(self.originalDate);

                if ($(e.delegateTarget).hasClass('open')) {
                    self.close();
                } else {
                    self._updateButtons();
                    self.open();
                }
            });

            // 취소 버튼 클릭시
            self.on('click', prefix + 'close', function (e) {
                e.preventDefault();

                self.close();
            });
            // 확인 버튼 클릭시
            self.on('click', prefix + 'confirm', function (e) {
                e.preventDefault();

                self.select();
                self.close();
            });

            // 이전년도 클릭
            self.on('click', prefix + 'prevbtn', function (e) {
                self.currentDate.setYear(self.currentDate.getFullYear() - 1);
                self._updateButtons();
            });

            // 다음년도 클릭
            self.on('click', prefix + 'nextbtn', function (e) {
                self.currentDate.setYear(self.currentDate.getFullYear() + 1);
                self._updateButtons();
            });

            // 월 선택
            self.on('click', prefix + 'month li>a', function (e) {
                e.preventDefault();

                self.currentDate.setMonth($(this).data('month') - 1);
                self.selectDate = core.clone(self.currentDate);

                // 버튼 선택
                self.select();
            });
        },
        /**
         * 달력열기
         */
        open: function open() {
            var self = this;

            self.triggerHandler('monthcalendarbeforeopen');
            self.$layer.stop().slideDown('fast', function () {
                self.$el.addClass('open');
                self.triggerHandler('monthcalendaropen');
                
                self.$('.ui_monthCalendar_toggle .hide').text('날짜 선택 닫기');

                $('body').vcGesture({
                    direction: 'vertical',
                    gesture: function (type, data) {
                        switch (type) {
                            case 'move':
                                data.event.preventDefault();
                                break;
                            case 'end':
                                if (data.direction === 'up') {
                                    self.close();
                                }
                        }
                    }
                });
            });
        },
        /**
         * 달력 닫기
         */
        close: function close() {
            var self = this;

            self.triggerHandler('monthcalendarbeforeclose');
            self.$layer.stop().slideUp('fast', function () {
                self.$el.removeClass('open');
                self.triggerHandler('monthcalendarclose');

                self.$('.ui_monthCalendar_toggle .hide').text('날짜 선택 열기');

                $('body').vcGesture('destroy');
            });
        },
        /**
         * 달력 선택
         */
        select: function select() {
            var self = this;

            self.triggerHandler('monthcalendarselect', {
                year: self.currentDate.getFullYear(),
                month: self.currentDate.getMonth() + 1
            });
        },
        /**
         * 버튼 문구, 상태 갱신
         * @private
         */
        _updateButtons: function _updateButtons() {
            var self = this;
            var currentYear = self.currentDate.getFullYear();

            if (currentYear <= self.startDate.getFullYear()) {
                self.$layer.find(prefix + 'prevbtn').prop('disabled', true);
            } else {
                self.$layer.find(prefix + 'prevbtn').prop('disabled', false);
            }

            if (currentYear >= (new Date()).getFullYear()) {
                self.$layer.find(prefix + 'nextbtn').prop('disabled', true);
            } else {
                self.$layer.find(prefix + 'nextbtn').prop('disabled', false);
            }

            self.$layer.find(prefix + 'prevtxt').text((currentYear - 1) + '년');
            self.$layer.find(prefix + 'curtxt').text(currentYear + '년');
            self.$layer.find(prefix + 'nexttxt').text((currentYear + 1) + '년');

            self._renderMonths();
        },
        _renderMonths: function _renderMonths() {
            var self = this;
            var html = '';
            var thisDate = new Date();
            var date = core.clone(self.currentDate);

            self.$list.empty();
            for (var m = 1, len = 12; m <= len; m++) {
                date.setMonth(m - 1);

                html += '<li>';
                if (!self._hasDate(date.getFullYear(), date.getMonth())) {
                    // 청구서가 없는 달
                    html += '<span class="dis" data-month="' + m + '">' + m + '월</span>';
                } else if (date.getFullYear() === self.selectDate.getFullYear()
                    && date.getMonth() === self.selectDate.getMonth()) {
                    // 선택된 달
                    html += '<a href="#" class="now" data-month="' + m + '">' + m + '월<span class="hide">선택됨</span></a>';
                } else {
                    // default
                    html += '<a href="#" class="on" data-month="' + m + '">' + m + '월</a>';
                }
                html += '</li>';
            }
            self.$list.html(html);
        },
        /**
         * 청구서가 있는 날인가...
         * @param year
         * @param month
         * @return {*|boolean}
         * @private
         */
        _hasDate: function (year, month) {
            return core.array.any(this.dates, function (date) {
                return date.getFullYear() === year && date.getMonth() === month;
            });
        },
        destroy: function destroy() {
            var self = this;

            self.$layer.off();
            self.supr();
        }
    });

    return MonthCalendar;
});